from PyQt5 import QtCore, QtGui
from gonha.util import Config
import psutil
import time
import humanfriendly
from gonha.util import Radeon
from gonha.util import Nvidia
from gonha.util import Weather
from gonha.util import Smart
from gonha.util import System
from unit_convert import UnitConvert
import portolan


class ThreadRadeon(QtCore.QThread):
    config = Config()
    tempType = config.getConfig('temptype')
    radeon = Radeon()
    signal = QtCore.pyqtSignal(list, name='ThreadRadeonFinish')
    message = []

    def __init__(self, parent=None):
        super(ThreadRadeon, self).__init__(parent)
        self.finished.connect(self.updateRadeon)

    def updateRadeon(self):
        self.message = self.radeon.get_radeon_gpus_info()
        self.update_memory_unit()
        self.updateTempWithUnit(self.config.getConfig('temptype'))
        self.signal.emit(self.message)
        self.start()

    def update_memory_unit(self):
        for i, gpu in enumerate(self.message):
            self.message[i]['memory_info']['vram_size'] = humanfriendly.format_size(gpu['memory_info']['vram_size'],
                                                                                    binary=True)
            self.message[i]['memory_used'] = humanfriendly.format_size(gpu['memory_used'], binary=True)

    def updateTempWithUnit(self, unit):
        if unit == 'Kelvin':
            for i, gpu in enumerate(self.message):
                self.message[i]['temperature_gpu'] = self.config.convertToKelvin(
                    gpu['temperature_gpu']
                )
                self.message[i]['temperature_gpu_high'] = self.config.convertToKelvin(
                    gpu['temperature_gpu_high']
                )
                self.message[i]['temperature_gpu_critical'] = self.config.convertToKelvin(
                    gpu['temperature_gpu_critical']
                )
                self.message[i]['temperature_scale'] = 'K'

        if unit == 'Fahrenheit':
            for i, gpu in enumerate(self.message):
                self.message[i]['temperature_gpu'] = self.config.convertToFahrenheit(
                    gpu['temperature_gpu']
                )
                self.message[i]['temperature_gpu_high'] = self.config.convertToFahrenheit(
                    gpu['temperature_gpu_high']
                )
                self.message[i]['temperature_gpu_critical'] = self.config.convertToFahrenheit(
                    gpu['temperature_gpu_critical']
                )
                self.message[i]['temperature_scale'] = 'F'

    def run(self):
        self.sleep(1)  # sleep for 1 sec


class ThreadNvidia(QtCore.QThread):
    config = Config()
    tempType = config.getConfig('temptype')
    nvidia = Nvidia()
    signal = QtCore.pyqtSignal(list, name='ThreadNvidiaFinish')
    message = []

    def __init__(self, parent=None):
        super(ThreadNvidia, self).__init__(parent)
        self.finished.connect(self.updateNvidia)

    def updateNvidia(self):
        self.message = self.nvidia.getGPUsInfo()
        self.updateTempWithUnit(self.config.getConfig('temptype'))
        self.signal.emit(self.message)
        self.start()

    def updateTempWithUnit(self, unit):
        if unit == 'Kelvin':
            for i, gpu in enumerate(self.message):
                self.message[i]['temperature_gpu'] = self.config.convertToKelvin(
                    gpu['temperature_gpu']
                )
                self.message[i]['temperature_gpu_high'] = self.config.convertToKelvin(
                    gpu['temperature_gpu_high']
                )
                self.message[i]['temperature_gpu_critical'] = self.config.convertToKelvin(
                    gpu['temperature_gpu_critical']
                )
                self.message[i]['temperature_scale'] = 'K'

        if unit == 'Fahrenheit':
            for i, gpu in enumerate(self.message):
                self.message[i]['temperature_gpu'] = self.config.convertToFahrenheit(
                    gpu['temperature_gpu']
                )
                self.message[i]['temperature_gpu_high'] = self.config.convertToFahrenheit(
                    gpu['temperature_gpu_high']
                )
                self.message[i]['temperature_gpu_critical'] = self.config.convertToFahrenheit(
                    gpu['temperature_gpu_critical']
                )
                self.message[i]['temperature_scale'] = 'F'

    def run(self):
        self.sleep(1)  # sleep for 1 sec


class ThreadWeather(QtCore.QThread):
    signal = QtCore.pyqtSignal(dict, name='ThreadWeatherFinish')
    weather = Weather()
    config = Config()

    def __init__(self, parent=None):
        super(ThreadWeather, self).__init__(parent)
        self.finished.connect(self.updateWeather)

    def updateWeather(self):
        tempType = self.config.getConfig('temptype')
        message = dict()
        if self.config.isOnline():
            try:
                data = self.weather.getData()
                scale = 'C'
                temp = float(data['main']['temp'])
                if tempType == 'Kelvin':
                    temp = self.config.convertToKelvin(temp)
                    scale = 'K'
                elif tempType == 'Fahrenheit':
                    temp = self.config.convertToFahrenheit(temp)
                    scale = 'F'

                # tempInteger = int(data['main']['temp'])
                message['temp'] = temp
                message['scale'] = scale
                message['humidity'] = f"{data['main']['humidity']}%"
                message['pressure'] = f"{data['main']['pressure']}hPa"
                visibilityAsKm = UnitConvert(metres=int(data['visibility'])).kilometres
                message['visibility'] = f"{visibilityAsKm}Km"
                windDir = portolan.abbr(float(data['wind']['deg']))
                message['wind'] = f"{data['wind']['speed']}m/s {windDir}"
                pixmap = QtGui.QPixmap()
                data = self.weather.getIcon(data['weather'][0]['icon'])
                pixmap.loadFromData(data)
                message['icon'] = pixmap
            except Exception as e:
                self.weather.printException(e)
                message.update({'temp': '', 'humidity': '', 'pressure': '', 'visibility': '', 'wind': ''})

            self.signal.emit(message)

        self.start()

    def run(self):
        self.sleep(1800)  # sleep 30 minutes


class ThreadNetworkStats(QtCore.QThread):
    signal = QtCore.pyqtSignal(dict, name='ThreadNetworkFinish')

    def __init__(self, parent=None):
        super(ThreadNetworkStats, self).__init__(parent)
        self.finished.connect(self.threadFinished)
        self.config = Config()
        self.iface = self.config.getConfig('iface')

    def threadFinished(self):
        self.start()

    def run(self):
        if self.config.isOnline():
            counter1 = psutil.net_io_counters(pernic=True)[self.iface]
            time.sleep(1)
            counter2 = psutil.net_io_counters(pernic=True)[self.iface]
            downSpeed = counter2.bytes_recv - counter1.bytes_recv
            upSpeed = counter2.bytes_sent - counter1.bytes_sent
            # get io statistics since boot
            net_io = psutil.net_io_counters(pernic=True)
            self.signal.emit(
                {
                    'downSpeed': downSpeed,
                    'upSpeed': upSpeed,
                    'iface': self.iface,
                    'bytesSent': net_io[self.iface].bytes_sent,
                    'bytesRcv': net_io[self.iface].bytes_recv
                }
            )


class ThreadSlow(QtCore.QThread):
    signal = QtCore.pyqtSignal(list, name='ThreadSlowFinish')

    def __init__(self, parent=None):
        super(ThreadSlow, self).__init__(parent)
        self.finished.connect(self.threadFinished)
        self.config = Config()

    def threadFinished(self):
        self.start()

    def getIpAddrs(self):
        if self.config.isOnline():
            ipDict = dict()
            ipDict['extip'] = self.config.getExtIp()
            ipDict['intip'] = self.config.getIntIp()
            ipDict['gw'] = self.config.getGw()
            return ipDict

    def getPartitions(self):
        msg = list()
        for mntPoint in self.config.getConfig('filesystems'):
            disk_usage = psutil.disk_usage(mntPoint)
            tempDict = dict()
            tempDict['mountpoint'] = mntPoint
            tempDict['total'] = '{}'.format(humanfriendly.format_size(disk_usage.total, binary=True))
            tempDict['used'] = '{}'.format(humanfriendly.format_size(disk_usage.used, binary=True))
            tempDict['free'] = '{}'.format(humanfriendly.format_size(disk_usage.free, binary=True))
            tempDict['percentUsed'] = disk_usage.percent
            tempDict['percentFree'] = 100 - int(disk_usage.percent)
            msg.append(tempDict)

        return msg

    def run(self):
        time.sleep(10)
        self.signal.emit(self.getPartitions())


class ThreadFast(QtCore.QThread):
    signal = QtCore.pyqtSignal(dict, name='ThreadFastFinish')
    system = System()

    def __init__(self, parent=None):
        super(ThreadFast, self).__init__(parent)
        self.finished.connect(self.threadFinished)

    def threadFinished(self):
        self.start()

    def run(self):
        self.signal.emit(self.system.getMessage())
        time.sleep(1)
